# -*- encoding: utf-8 -*-
require 'spec_helper'
#色地

describe GroundColorsController do
  before do
    SpeechBalloonTemplate.delete_all
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group, :name => 'peta'
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end

if MagicNumber['run_mode'] == 1
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @gc = FactoryGirl.create :ground_color
      GroundColor.stub(:list).and_return([@gc, @gc, @gc])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq GroundColor.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq GroundColor.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq GroundColor.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '色地モデルに一覧を問い合わせている' do
        GroundColor.should_receive(:list).exactly(1)
        get :index
      end
      it '@ground_colorsにリストを取得している' do
        get :index
        assigns(:ground_colors).should have_at_least(3).items
      end
      context 'html形式' do
        it '@paginateにページ制御を取得している' do
          get :index
          assigns(:paginate).is_a?(Kaminari::PaginatableArray).should be_true
        end
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '色地モデルにjson一覧出力オプションを問い合わせている' do
          GroundColor.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは色地っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("panel_id").should be_true
          json.first.has_key?("code").should be_true
          json.first.has_key?("z").should be_true
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @gc = FactoryGirl.create :ground_color
      GroundColor.stub(:show).and_return(@gc)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @gc.id
        response.should be_success
      end
      it '色地モデルに単体取得を問い合わせている' do
        GroundColor.should_receive(:show).exactly(1)
        get :show
      end
      it '@ground_colorにアレを取得している' do
        get :show, :id => @gc.id
        assigns(:ground_color).should eq(@gc)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @gc.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @gc.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '色地モデルにjson単体出力オプションを問い合わせている' do
          GroundColor.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @gc.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @gc.id, :format => :json
          json = JSON.parse response.body
          json["panel_id"].should_not be_nil
          json["z"].should_not be_nil
          json["code"].should_not be_nil
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @gc.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @gc.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @gc.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @gc.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @gc.id
        response.should be_success 
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @gc.id
        response.should be_success 
      end
    end
  end
  
else
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @gc = FactoryGirl.create :ground_color
      GroundColor.stub(:list).and_return([@gc, @gc, @gc])
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @gc = FactoryGirl.create :ground_color
      GroundColor.stub(:show).and_return(@gc)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @gc.id
        response.should be_success
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @gc.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @gc.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @gc.id
        response.should be_success
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @gc.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @gc.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
end
end
