module ControllerMacros
  def sign_in_url
    '/users/sign_in'
  end
  
  def return_ok?
    # ステータス OKを返す
    expect(response).to be_success
  end
  
  def render_json?
    # jsonデータを返す
    expect {JSON.parse(response.body)}.not_to raise_error
  end
  
  def render_filer?
    # filerテンプレートを描画する
    expect(response).to render_template("templates/r/filer/filer")
  end
  
  def render_profiler?
    # profilerテンプレートを描画する
    expect(response).to render_template("templates/r/profiler/profiler")
  end
  
  def render_form?
    # formテンプレートを描画する
    expect(response).to render_template("templates/r/form/form")
  end
  
  def redirect? url
    # html形式: ステータスコード302 Foundを返す
    expect(response.status).to eq(302)
    expect(response).to redirect_to url
  end
  
  # ゲストがapiからコンテンツにアクセスしようとした時の動作を確認する
  def render_unauthorized?
    # json形式: ステータスコード401 Unauthorizedを返す
    expect(response.status).to eq(401)
    expect(response.message).to match(/Unauthorized/)
  end
  
  # 処理不能を返す
  def render_unprocessable?
    expect(response.status).to eq(422)
    expect(response.message).to match(/Unprocessable/)
  end
  
  def render_complated?
    expect(response.message).to match /./
  end
  
  def set_model? model_class, var_name = :my_model_class
    expect(assigns(var_name)).to be model_class
  end
  
  def set_list_model? model_class, var_name = :my_list_model_class
    expect(assigns(var_name)).to be model_class
  end
  
  def set_list? item_name, list_name, var_name = :list
    expect(assigns(var_name).item_name).to eq(item_name)
    expect(assigns(var_name).list_name).to eq(list_name)
  end
  
  def set_play_list? model_class
    expect(assigns(:items).size).to be >= 3
    expect(assigns(:count)).to be >= 3
    expect(assigns(:pager)).not_to be nil
    expect(assigns(:items).first.class).to be model_class
  end
  
  def let_filer? var_name = :filer
    expect(assigns(var_name).class).to eq(Locmare::Filer)
  end
  
  def let_profiler? var_name = :profiler
    expect(assigns(var_name).class).to eq(Locmare::Profiler)
  end
  
  def set_bucket? item, var_name = :form
    # @filerにファイラーを取得している
    expect(assigns(var_name).item).to eq(item)
    expect(assigns(var_name).mounted).to eq(true)
    expect(assigns(var_name).submit).to eq(true)
  end
  
  def boost? names, var_name = :item
    if names.empty?
      assigns(var_name).boosters.empty?
    else
      names.each do |boost_name|
        expect(assigns(var_name).boosters[boost_name]).to_not be nil
      end
    end
  end
  
  # 作家活動していないユーザーがAPIから創作活動をしようとしたとき、例外を発生させる。
  # 暫定的な仕様なので、良い方法があれば変えるかもしれない。メンテしやすいように切り分けてある。
  # リクエストして例外が発生したところとらえるので、リクエストと検証を同時に行う
  def announce_regist_author_for_json method, params = {}, action = :create
    expect {__send__(method, action, params.merge(:format => :json))}.to raise_error(ActiveRecord::Forbidden)
  end
  
  # 保存処理を成功したことにしてスキップする
  def allow_save my_model_class
    allow_any_instance_of(my_model_class).to receive(:save).and_return(true)
  end
  
  # 保存処理を失敗したことにしてスキップする
  def reject_save my_model_class
    allow_any_instance_of(my_model_class).to receive(:save).and_return(false)
  end
  
  # 削除処理を成功したことにしてスキップする
  def allow_destroy my_model_class
    allow_any_instance_of(my_model_class).to receive(:destroy).and_return(true)
  end
  
  # 削除処理を失敗したことにしてスキップする
  def reject_destroy my_model_class
    allow_any_instance_of(my_model_class).to receive(:destroy).and_return(false)
  end
  
  def factory_contents factory_name, author, count = 5
    FactoryGirl.create_list(factory_name, count, author: author)
  end
  
end

